body {
    margin-top: 0;
    background-color: #fbf5d5; 
}

body * {
    font-family: georgia, verdana, arial, sans-serif;
}

h2 {
    margin-left: 0;
}

p {
    padding: 0.4em;
    margin-bottom: 0;
}

a {
    text-decoration: none;
    color: #444488;
}
a:visited {
    color: #660066;
}

label { font-weight: bold; }

#top-bar {
    position: relative;
    top: 0; left: 0;
    padding: 0;
    margin: 1ex 0;
    padding-bottom: 1ex;
    width: 100%;

    border-bottom: 1px solid;

    font-size: 16px;
}
#top-bar * { font-size: 16px; }
#top-bar h1 {
    display: inline;
    margin-top: 0;
    padding-top: 0;

    font-weight: normal;
}

#logo img {
    padding-right: 0.3em;
    border: 0;
}
#navigation-bar {
    position: absolute;
    top: 0;
    right: 0;
}
#content { clear: both; }

#search-form { display: inline; text-align: right; }
#search-form-contents { display: inline }

.rank {
    float: left;
    font-size: large;
}
.result {
    margin-top: 2ex;
}
.result-description {
    font-size: large;
    margin-left: 4em;
    display: block;
}
.extract {
    font-size: medium;
    font-style: italic;
    display: block;
}

.pager {
#    text-align: right;
    border-top: 0;
    margin-top: 0;
}

.page  {
    font-weight: bold;
    font-size: 100%;
}
.current-page { color: #bbb }

.matched-fragment {
    background-color: #ffff66;
}

#dir-panel {
    margin: 1em;
}
.dir-panel-entry-subsection { margin-left: 1em; }

.doc_name {
    color: #333366;
    font-weight: bold;
}
.doc_description {
    font-size: 100%;
    margin-top: 0.3em;
    margin-bottom: 2ex;
}

#item-list {
    float: right;
    width: 80%;
}

.spacer {
    padding-top: 0;
    padding-left: 1ex;
    padding-right: 6ex;
    padding-botom: 1ex;
    width: 96%;
}

.disabled {
    color: gray;
}

.note {
    width: 90%;
    color: #666;
    background-color: #fbe2cb;
    padding: 1ex;
    font-size: 80%;
    font-weight: bold;
    margin-bottom: 1ex;
}
